@@ -100,9 +100,7 @@ class TestFillFixtures:
         sub1.join("test_in_sub1.py").write("def test_1(arg1): pass")
         sub2.join("test_in_sub2.py").write("def test_2(arg2): pass")
         result = testdir.runpytest("-v")
-        result.stdout.fnmatch_lines([
-            "*2 passed*"
-        ])
+        result.assert_outcomes(passed=2)
 
     def test_extend_fixture_module_class(self, testdir):
         testfile = testdir.makepyfile("""
@@ -557,7 +555,8 @@ class TestRequestBasic:
                 pass
             def test_function(request, farg):
                 assert set(get_public_names(request.fixturenames)) == \
-                       set(["tmpdir", "sarg", "arg1", "request", "farg"])
+                       set(["tmpdir", "sarg", "arg1", "request", "farg",
+                            "tmpdir_factory"])
         """)
         reprec = testdir.inline_run()
         reprec.assertoutcome(passed=1)
@@ -1492,7 +1491,7 @@ class TestAutouseManagement:
         reprec = testdir.inline_run("-v","-s")
         reprec.assertoutcome(passed=8)
         config = reprec.getcalls("pytest_unconfigure")[0].config
-        l = config._conftest.getconftestmodules(p)[0].l
+        l = config.pluginmanager._getconftestmodules(p)[0].l
         assert l == ["fin_a1", "fin_a2", "fin_b1", "fin_b2"] * 2
 
     def test_scope_ordering(self, testdir):
@@ -1599,6 +1598,42 @@ class TestFixtureMarker:
         reprec = testdir.inline_run()
         reprec.assertoutcome(passed=4)
 
+    def test_multiple_parametrization_issue_736(self, testdir):
+        testdir.makepyfile("""
+            import pytest
+
+            @pytest.fixture(params=[1,2,3])
+            def foo(request):
+                return request.param
+
+            @pytest.mark.parametrize('foobar', [4,5,6])
+            def test_issue(foo, foobar):
+                assert foo in [1,2,3]
+                assert foobar in [4,5,6]
+        """)
+        reprec = testdir.inline_run()
+        reprec.assertoutcome(passed=9)
+
+    @pytest.mark.parametrize('param_args', ["'fixt, val'", "'fixt,val'", "['fixt', 'val']", "('fixt', 'val')"])
+    def test_override_parametrized_fixture_issue_979(self, testdir, param_args):
+        """Make sure a parametrized argument can override a parametrized fixture.
+
+        This was a regression introduced in the fix for #736.
+        """
+        testdir.makepyfile("""
+            import pytest
+
+            @pytest.fixture(params=[1, 2])
+            def fixt(request):
+                return request.param
+
+            @pytest.mark.parametrize(%s, [(3, 'x'), (4, 'x')])
+            def test_foo(fixt, val):
+                pass
+        """ % param_args)
+        reprec = testdir.inline_run()
+        reprec.assertoutcome(passed=2)
+
     def test_scope_session(self, testdir):
         testdir.makepyfile("""
             import pytest
