@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*-
 import re
 
 import pytest, py
@@ -118,6 +119,41 @@ class TestMetafunc:
         assert metafunc._calls[2].id == "x1-a"
         assert metafunc._calls[3].id == "x1-b"
 
+    @pytest.mark.skipif('sys.version_info[0] >= 3')
+    def test_unicode_idval_python2(self):
+        """unittest for the expected behavior to obtain ids for parametrized
+        unicode values in Python 2: if convertible to ascii, they should appear
+        as ascii values, otherwise fallback to hide the value behind the name
+        of the parametrized variable name. #1086
+        """
+        from _pytest.python import _idval
+        values = [
+            (u'', ''),
+            (u'ascii', 'ascii'),
+            (u'ação', 'a6'),
+            (u'josé@blah.com', 'a6'),
+            (u'δοκ.ιμή@παράδειγμα.δοκιμή', 'a6'),
+        ]
+        for val, expected in values:
+            assert _idval(val, 'a', 6, None) == expected
+
+    def test_bytes_idval(self):
+        """unittest for the expected behavior to obtain ids for parametrized
+        bytes values:
+        - python2: non-ascii strings are considered bytes and formatted using
+        "binary escape", where any byte < 127 is escaped into its hex form.
+        - python3: bytes objects are always escaped using "binary escape".
+        """
+        from _pytest.python import _idval
+        values = [
+            (b'', ''),
+            (b'\xc3\xb4\xff\xe4', '\\xc3\\xb4\\xff\\xe4'),
+            (b'ascii', 'ascii'),
+            (u'αρά'.encode('utf-8'), '\\xce\\xb1\\xcf\\x81\\xce\\xac'),
+        ]
+        for val, expected in values:
+            assert _idval(val, 'a', 6, None) == expected
+
     @pytest.mark.issue250
     def test_idmaker_autoname(self):
         from _pytest.python import idmaker
