@@ -231,6 +231,17 @@ class TestAssert_reprcompare:
         assert expl[1] == py.builtin._totext('- £€', 'utf-8')
         assert expl[2] == py.builtin._totext('+ £', 'utf-8')
 
+    def test_nonascii_text(self):
+        """
+        :issue: 877
+        non ascii python2 str caused a UnicodeDecodeError
+        """
+        class A(str):
+            def __repr__(self):
+                return '\xff'
+        expl = callequal(A(), '1')
+        assert expl
+
     def test_mojibake(self):
         # issue 429
         left = 'e'
@@ -411,8 +422,13 @@ def test_assert_compare_truncate_longmessage(testdir):
     """)
 
     result = testdir.runpytest()
+    # without -vv, truncate the message showing a few diff lines only
     result.stdout.fnmatch_lines([
-        "*truncated*use*-vv*",
+        "*- 1",
+        "*- 3",
+        "*- 5",
+        "*- 7",
+        "*truncated (191 more lines)*use*-vv*",
     ])
 
 
@@ -461,7 +477,7 @@ def test_assertion_options(testdir):
                    ("--assert=plain", "--nomagic"),
                    ("--assert=plain", "--no-assert", "--nomagic"))
     for opt in off_options:
-        result = testdir.runpytest(*opt)
+        result = testdir.runpytest_subprocess(*opt)
         assert "3 == 4" not in result.stdout.str()
 
 def test_old_assert_mode(testdir):
@@ -469,7 +485,7 @@ def test_old_assert_mode(testdir):
         def test_in_old_mode():
             assert "@py_builtins" not in globals()
     """)
-    result = testdir.runpytest("--assert=reinterp")
+    result = testdir.runpytest_subprocess("--assert=reinterp")
     assert result.ret == 0
 
 def test_triple_quoted_string_issue113(testdir):
