@@ -1,3 +1,4 @@
+# encoding: utf-8
 import sys
 from _pytest.doctest import DoctestItem, DoctestModule, DoctestTextfile
 import py
@@ -109,6 +110,46 @@ class TestDoctests:
             "*UNEXPECTED*ZeroDivision*",
         ])
 
+    def test_docstring_context_around_error(self, testdir):
+        """Test that we show some context before the actual line of a failing
+        doctest.
+        """
+        testdir.makepyfile('''
+            def foo():
+                """
+                text-line-1
+                text-line-2
+                text-line-3
+                text-line-4
+                text-line-5
+                text-line-6
+                text-line-7
+                text-line-8
+                text-line-9
+                text-line-10
+                text-line-11
+                >>> 1 + 1
+                3
+
+                text-line-after
+                """
+        ''')
+        result = testdir.runpytest('--doctest-modules')
+        result.stdout.fnmatch_lines([
+            '*docstring_context_around_error*',
+            '005*text-line-3',
+            '006*text-line-4',
+            '013*text-line-11',
+            '014*>>> 1 + 1',
+            'Expected:',
+            '    3',
+            'Got:',
+            '    2',
+        ])
+        # lines below should be trimmed out
+        assert 'text-line-2' not in result.stdout.str()
+        assert 'text-line-after' not in result.stdout.str()
+
     def test_doctest_linedata_missing(self, testdir):
         testdir.tmpdir.join('hello.py').write(py.code.Source("""
             class Fun(object):
@@ -339,6 +380,23 @@ class TestDoctests:
         reprec = testdir.inline_run(p, "--doctest-glob=x*.txt")
         reprec.assertoutcome(failed=1, passed=0)
 
+    def test_contains_unicode(self, testdir):
+        """Fix internal error with docstrings containing non-ascii characters.
+        """
+        testdir.makepyfile(u'''
+            # encoding: utf-8
+            def foo():
+                """
+                >>> name = 'с' # not letter 'c' but instead Cyrillic 's'.
+                'anything'
+                """
+        ''')
+        result = testdir.runpytest('--doctest-modules')
+        result.stdout.fnmatch_lines([
+            'Got nothing',
+            '* 1 failed in*',
+        ])
+
     def test_ignore_import_errors_on_doctest(self, testdir):
         p = testdir.makepyfile("""
             import asdf
@@ -371,38 +429,6 @@ class TestDoctests:
                                     "--junit-xml=junit.xml")
         reprec.assertoutcome(failed=1)
 
-    def test_doctest_module_session_fixture(self, testdir):
-        """Test that session fixtures are initialized for doctest modules (#768)
-        """
-        # session fixture which changes some global data, which will
-        # be accessed by doctests in a module
-        testdir.makeconftest("""
-            import pytest
-            import sys
-
-            @pytest.yield_fixture(autouse=True, scope='session')
-            def myfixture():
-                assert not hasattr(sys, 'pytest_session_data')
-                sys.pytest_session_data = 1
-                yield
-                del sys.pytest_session_data
-        """)
-        testdir.makepyfile(foo="""
-            import sys
-
-            def foo():
-              '''
-              >>> assert sys.pytest_session_data == 1
-              '''
-
-            def bar():
-              '''
-              >>> assert sys.pytest_session_data == 1
-              '''
-        """)
-        result = testdir.runpytest("--doctest-modules")
-        result.stdout.fnmatch_lines('*2 passed*')
-
     @pytest.mark.parametrize('config_mode', ['ini', 'comment'])
     def test_allow_unicode(self, testdir, config_mode):
         """Test that doctests which output unicode work in all python versions
@@ -446,7 +472,7 @@ class TestDoctests:
         reprec.assertoutcome(passed=passed, failed=int(not passed))
 
 
-class TestDocTestSkips:
+class TestDoctestSkips:
     """
     If all examples in a doctest are skipped due to the SKIP option, then
     the tests should be SKIPPED rather than PASSED. (#957)
@@ -493,3 +519,122 @@ class TestDocTestSkips:
         """)
         reprec = testdir.inline_run("--doctest-modules")
         reprec.assertoutcome(skipped=1)
+
+
+class TestDoctestAutoUseFixtures:
+
+    SCOPES = ['module', 'session', 'class', 'function']
+
+    def test_doctest_module_session_fixture(self, testdir):
+        """Test that session fixtures are initialized for doctest modules (#768)
+        """
+        # session fixture which changes some global data, which will
+        # be accessed by doctests in a module
+        testdir.makeconftest("""
+            import pytest
+            import sys
+
+            @pytest.yield_fixture(autouse=True, scope='session')
+            def myfixture():
+                assert not hasattr(sys, 'pytest_session_data')
+                sys.pytest_session_data = 1
+                yield
+                del sys.pytest_session_data
+        """)
+        testdir.makepyfile(foo="""
+            import sys
+
+            def foo():
+              '''
+              >>> assert sys.pytest_session_data == 1
+              '''
+
+            def bar():
+              '''
+              >>> assert sys.pytest_session_data == 1
+              '''
+        """)
+        result = testdir.runpytest("--doctest-modules")
+        result.stdout.fnmatch_lines('*2 passed*')
+
+    @pytest.mark.parametrize('scope', SCOPES)
+    @pytest.mark.parametrize('enable_doctest', [True, False])
+    def test_fixture_scopes(self, testdir, scope, enable_doctest):
+        """Test that auto-use fixtures work properly with doctest modules.
+        See #1057 and #1100.
+        """
+        testdir.makeconftest('''
+            import pytest
+
+            @pytest.fixture(autouse=True, scope="{scope}")
+            def auto(request):
+                return 99
+        '''.format(scope=scope))
+        testdir.makepyfile(test_1='''
+            def test_foo():
+                """
+                >>> getfixture('auto') + 1
+                100
+                """
+            def test_bar():
+                assert 1
+        ''')
+        params = ('--doctest-modules',) if enable_doctest else ()
+        passes = 3 if enable_doctest else 2
+        result = testdir.runpytest(*params)
+        result.stdout.fnmatch_lines(['*=== %d passed in *' % passes])
+
+    @pytest.mark.parametrize('scope', SCOPES)
+    @pytest.mark.parametrize('autouse', [True, False])
+    @pytest.mark.parametrize('use_fixture_in_doctest', [True, False])
+    def test_fixture_module_doctest_scopes(self, testdir, scope, autouse,
+                                           use_fixture_in_doctest):
+        """Test that auto-use fixtures work properly with doctest files.
+        See #1057 and #1100.
+        """
+        testdir.makeconftest('''
+            import pytest
+
+            @pytest.fixture(autouse={autouse}, scope="{scope}")
+            def auto(request):
+                return 99
+        '''.format(scope=scope, autouse=autouse))
+        if use_fixture_in_doctest:
+            testdir.maketxtfile(test_doc="""
+                >>> getfixture('auto')
+                99
+            """)
+        else:
+            testdir.maketxtfile(test_doc="""
+                >>> 1 + 1
+                2
+            """)
+        result = testdir.runpytest('--doctest-modules')
+        assert 'FAILURES' not in str(result.stdout.str())
+        result.stdout.fnmatch_lines(['*=== 1 passed in *'])
+
+    @pytest.mark.parametrize('scope', SCOPES)
+    def test_auto_use_request_attributes(self, testdir, scope):
+        """Check that all attributes of a request in an autouse fixture
+        behave as expected when requested for a doctest item.
+        """
+        testdir.makeconftest('''
+            import pytest
+
+            @pytest.fixture(autouse=True, scope="{scope}")
+            def auto(request):
+                if "{scope}" == 'module':
+                    assert request.module is None
+                if "{scope}" == 'class':
+                    assert request.cls is None
+                if "{scope}" == 'function':
+                    assert request.function is None
+                return 99
+        '''.format(scope=scope))
+        testdir.maketxtfile(test_doc="""
+            >>> 1 + 1
+            2
+        """)
+        result = testdir.runpytest('--doctest-modules')
+        assert 'FAILURES' not in str(result.stdout.str())
+        result.stdout.fnmatch_lines(['*=== 1 passed in *'])
