@@ -2,7 +2,9 @@
 
 from xml.dom import minidom
 from _pytest.main import EXIT_NOTESTSCOLLECTED
-import py, sys, os
+import py
+import sys
+import os
 from _pytest.junitxml import LogXML
 import pytest
 
@@ -11,16 +13,72 @@ def runandparse(testdir, *args):
     resultpath = testdir.tmpdir.join("junit.xml")
     result = testdir.runpytest("--junitxml=%s" % resultpath, *args)
     xmldoc = minidom.parse(str(resultpath))
-    return result, xmldoc
+    return result, DomNode(xmldoc)
+
 
 def assert_attr(node, **kwargs):
     __tracebackhide__ = True
-    for name, expected in kwargs.items():
+
+    def nodeval(node, name):
         anode = node.getAttributeNode(name)
-        assert anode, "node %r has no attribute %r" %(node, name)
-        val = anode.value
-        if val != str(expected):
-            py.test.fail("%r != %r" %(str(val), str(expected)))
+        if anode is not None:
+            return anode.value
+
+    expected = dict((name, str(value)) for name, value in kwargs.items())
+    on_node = dict((name, nodeval(node, name)) for name in expected)
+    assert on_node == expected
+
+
+class DomNode(object):
+    def __init__(self, dom):
+        self.__node = dom
+
+    def __repr__(self):
+        return self.__node.toxml()
+
+    def find_first_by_tag(self, tag):
+        return self.find_nth_by_tag(tag, 0)
+
+    def _by_tag(self, tag):
+        return self.__node.getElementsByTagName(tag)
+
+    def find_nth_by_tag(self, tag, n):
+        items = self._by_tag(tag)
+        try:
+            nth = items[n]
+        except IndexError:
+            pass
+        else:
+            return type(self)(nth)
+
+    def find_by_tag(self, tag):
+        t = type(self)
+        return [t(x) for x in self.__node.getElementsByTagName(tag)]
+
+    def __getitem__(self, key):
+        node = self.__node.getAttributeNode(key)
+        if node is not None:
+            return node.value
+
+    def assert_attr(self, **kwargs):
+        __tracebackhide__ = True
+        return assert_attr(self.__node, **kwargs)
+
+    def toxml(self):
+        return self.__node.toxml()
+
+    @property
+    def text(self):
+        return self.__node.childNodes[0].wholeText
+
+    @property
+    def tag(self):
+        return self.__node.tagName
+
+    @property
+    def next_siebling(self):
+        return type(self)(self.__node.nextSibling)
+
 
 class TestPython:
     def test_summing_simple(self, testdir):
@@ -41,8 +99,8 @@ class TestPython:
         """)
         result, dom = runandparse(testdir)
         assert result.ret
-        node = dom.getElementsByTagName("testsuite")[0]
-        assert_attr(node, name="pytest", errors=0, failures=1, skips=3, tests=2)
+        node = dom.find_first_by_tag("testsuite")
+        node.assert_attr(name="pytest", errors=0, failures=1, skips=3, tests=2)
 
     def test_timing_function(self, testdir):
         testdir.makepyfile("""
@@ -55,9 +113,9 @@ class TestPython:
                 time.sleep(0.01)
         """)
         result, dom = runandparse(testdir)
-        node = dom.getElementsByTagName("testsuite")[0]
-        tnode = node.getElementsByTagName("testcase")[0]
-        val = tnode.getAttributeNode("time").value
+        node = dom.find_first_by_tag("testsuite")
+        tnode = node.find_first_by_tag("testcase")
+        val = tnode["time"]
         assert round(float(val), 2) >= 0.03
 
     def test_setup_error(self, testdir):
@@ -69,16 +127,16 @@ class TestPython:
         """)
         result, dom = runandparse(testdir)
         assert result.ret
-        node = dom.getElementsByTagName("testsuite")[0]
-        assert_attr(node, errors=1, tests=0)
-        tnode = node.getElementsByTagName("testcase")[0]
-        assert_attr(tnode,
+        node = dom.find_first_by_tag("testsuite")
+        node.assert_attr(errors=1, tests=0)
+        tnode = node.find_first_by_tag("testcase")
+        tnode.assert_attr(
             file="test_setup_error.py",
             line="2",
             classname="test_setup_error",
             name="test_function")
-        fnode = tnode.getElementsByTagName("error")[0]
-        assert_attr(fnode, message="test setup failure")
+        fnode = tnode.find_first_by_tag("error")
+        fnode.assert_attr(message="test setup failure")
         assert "ValueError" in fnode.toxml()
 
     def test_skip_contains_name_reason(self, testdir):
@@ -89,19 +147,16 @@ class TestPython:
         """)
         result, dom = runandparse(testdir)
         assert result.ret == 0
-        node = dom.getElementsByTagName("testsuite")[0]
-        assert_attr(node, skips=1)
-        tnode = node.getElementsByTagName("testcase")[0]
-        assert_attr(tnode,
+        node = dom.find_first_by_tag("testsuite")
+        node.assert_attr(skips=1)
+        tnode = node.find_first_by_tag("testcase")
+        tnode.assert_attr(
             file="test_skip_contains_name_reason.py",
             line="1",
             classname="test_skip_contains_name_reason",
             name="test_skip")
-        snode = tnode.getElementsByTagName("skipped")[0]
-        assert_attr(snode,
-            type="pytest.skip",
-            message="hello23",
-            )
+        snode = tnode.find_first_by_tag("skipped")
+        snode.assert_attr(type="pytest.skip", message="hello23", )
 
     def test_classname_instance(self, testdir):
         testdir.makepyfile("""
@@ -111,10 +166,10 @@ class TestPython:
         """)
         result, dom = runandparse(testdir)
         assert result.ret
-        node = dom.getElementsByTagName("testsuite")[0]
-        assert_attr(node, failures=1)
-        tnode = node.getElementsByTagName("testcase")[0]
-        assert_attr(tnode,
+        node = dom.find_first_by_tag("testsuite")
+        node.assert_attr(failures=1)
+        tnode = node.find_first_by_tag("testcase")
+        tnode.assert_attr(
             file="test_classname_instance.py",
             line="1",
             classname="test_classname_instance.TestClass",
@@ -125,10 +180,10 @@ class TestPython:
         p.write("def test_func(): 0/0")
         result, dom = runandparse(testdir)
         assert result.ret
-        node = dom.getElementsByTagName("testsuite")[0]
-        assert_attr(node, failures=1)
-        tnode = node.getElementsByTagName("testcase")[0]
-        assert_attr(tnode,
+        node = dom.find_first_by_tag("testsuite")
+        node.assert_attr(failures=1)
+        tnode = node.find_first_by_tag("testcase")
+        tnode.assert_attr(
             file=os.path.join("sub", "test_hello.py"),
             line="0",
             classname="sub.test_hello",
@@ -139,12 +194,12 @@ class TestPython:
         testdir.makepyfile("def test_function(): pass")
         result, dom = runandparse(testdir)
         assert result.ret
-        node = dom.getElementsByTagName("testsuite")[0]
-        assert_attr(node, errors=1, tests=0)
-        tnode = node.getElementsByTagName("testcase")[0]
-        assert_attr(tnode, classname="pytest", name="internal")
-        fnode = tnode.getElementsByTagName("error")[0]
-        assert_attr(fnode, message="internal error")
+        node = dom.find_first_by_tag("testsuite")
+        node.assert_attr(errors=1, tests=0)
+        tnode = node.find_first_by_tag("testcase")
+        tnode.assert_attr(classname="pytest", name="internal")
+        fnode = tnode.find_first_by_tag("error")
+        fnode.assert_attr(message="internal error")
         assert "Division" in fnode.toxml()
 
     def test_failure_function(self, testdir):
@@ -158,22 +213,22 @@ class TestPython:
 
         result, dom = runandparse(testdir)
         assert result.ret
-        node = dom.getElementsByTagName("testsuite")[0]
-        assert_attr(node, failures=1, tests=1)
-        tnode = node.getElementsByTagName("testcase")[0]
-        assert_attr(tnode,
+        node = dom.find_first_by_tag("testsuite")
+        node.assert_attr(failures=1, tests=1)
+        tnode = node.find_first_by_tag("testcase")
+        tnode.assert_attr(
             file="test_failure_function.py",
             line="1",
             classname="test_failure_function",
             name="test_fail")
-        fnode = tnode.getElementsByTagName("failure")[0]
-        assert_attr(fnode, message="ValueError: 42")
+        fnode = tnode.find_first_by_tag("failure")
+        fnode.assert_attr(message="ValueError: 42")
         assert "ValueError" in fnode.toxml()
-        systemout = fnode.nextSibling
-        assert systemout.tagName == "system-out"
+        systemout = fnode.next_siebling
+        assert systemout.tag == "system-out"
         assert "hello-stdout" in systemout.toxml()
-        systemerr = systemout.nextSibling
-        assert systemerr.tagName == "system-err"
+        systemerr = systemout.next_siebling
+        assert systemerr.tag == "system-err"
         assert "hello-stderr" in systemerr.toxml()
 
     def test_failure_verbose_message(self, testdir):
@@ -184,10 +239,10 @@ class TestPython:
         """)
 
         result, dom = runandparse(testdir)
-        node = dom.getElementsByTagName("testsuite")[0]
-        tnode = node.getElementsByTagName("testcase")[0]
-        fnode = tnode.getElementsByTagName("failure")[0]
-        assert_attr(fnode, message="AssertionError: An error assert 0")
+        node = dom.find_first_by_tag("testsuite")
+        tnode = node.find_first_by_tag("testcase")
+        fnode = tnode.find_first_by_tag("failure")
+        fnode.assert_attr(message="AssertionError: An error assert 0")
 
     def test_failure_escape(self, testdir):
         testdir.makepyfile("""
@@ -199,22 +254,21 @@ class TestPython:
         """)
         result, dom = runandparse(testdir)
         assert result.ret
-        node = dom.getElementsByTagName("testsuite")[0]
-        assert_attr(node, failures=3, tests=3)
+        node = dom.find_first_by_tag("testsuite")
+        node.assert_attr(failures=3, tests=3)
 
         for index, char in enumerate("<&'"):
 
-            tnode = node.getElementsByTagName("testcase")[index]
-            assert_attr(tnode,
+            tnode = node.find_nth_by_tag("testcase", index)
+            tnode.assert_attr(
                 file="test_failure_escape.py",
                 line="1",
                 classname="test_failure_escape",
                 name="test_func[%s]" % char)
-            sysout = tnode.getElementsByTagName('system-out')[0]
-            text = sysout.childNodes[0].wholeText
+            sysout = tnode.find_first_by_tag('system-out')
+            text = sysout.text
             assert text == '%s\n' % char
 
-
     def test_junit_prefixing(self, testdir):
         testdir.makepyfile("""
             def test_func():
@@ -225,20 +279,20 @@ class TestPython:
         """)
         result, dom = runandparse(testdir, "--junitprefix=xyz")
         assert result.ret
-        node = dom.getElementsByTagName("testsuite")[0]
-        assert_attr(node, failures=1, tests=2)
-        tnode = node.getElementsByTagName("testcase")[0]
-        assert_attr(tnode,
+        node = dom.find_first_by_tag("testsuite")
+        node.assert_attr(failures=1, tests=2)
+        tnode = node.find_first_by_tag("testcase")
+        tnode.assert_attr(
             file="test_junit_prefixing.py",
             line="0",
             classname="xyz.test_junit_prefixing",
             name="test_func")
-        tnode = node.getElementsByTagName("testcase")[1]
-        assert_attr(tnode,
+        tnode = node.find_nth_by_tag("testcase", 1)
+        tnode.assert_attr(
             file="test_junit_prefixing.py",
             line="3",
             classname="xyz.test_junit_prefixing."
-                      "TestHello",
+            "TestHello",
             name="test_hello")
 
     def test_xfailure_function(self, testdir):
@@ -249,17 +303,17 @@ class TestPython:
         """)
         result, dom = runandparse(testdir)
         assert not result.ret
-        node = dom.getElementsByTagName("testsuite")[0]
-        assert_attr(node, skips=1, tests=0)
-        tnode = node.getElementsByTagName("testcase")[0]
-        assert_attr(tnode,
+        node = dom.find_first_by_tag("testsuite")
+        node.assert_attr(skips=1, tests=0)
+        tnode = node.find_first_by_tag("testcase")
+        tnode.assert_attr(
             file="test_xfailure_function.py",
             line="1",
             classname="test_xfailure_function",
             name="test_xfail")
-        fnode = tnode.getElementsByTagName("skipped")[0]
-        assert_attr(fnode, message="expected test failure")
-        #assert "ValueError" in fnode.toxml()
+        fnode = tnode.find_first_by_tag("skipped")
+        fnode.assert_attr(message="expected test failure")
+        # assert "ValueError" in fnode.toxml()
 
     def test_xfailure_xpass(self, testdir):
         testdir.makepyfile("""
@@ -269,49 +323,50 @@ class TestPython:
                 pass
         """)
         result, dom = runandparse(testdir)
-        #assert result.ret
-        node = dom.getElementsByTagName("testsuite")[0]
-        assert_attr(node, skips=1, tests=0)
-        tnode = node.getElementsByTagName("testcase")[0]
-        assert_attr(tnode,
+        # assert result.ret
+        node = dom.find_first_by_tag("testsuite")
+        node.assert_attr(skips=1, tests=0)
+        tnode = node.find_first_by_tag("testcase")
+        tnode.assert_attr(
             file="test_xfailure_xpass.py",
             line="1",
             classname="test_xfailure_xpass",
             name="test_xpass")
-        fnode = tnode.getElementsByTagName("skipped")[0]
-        assert_attr(fnode, message="xfail-marked test passes unexpectedly")
-        #assert "ValueError" in fnode.toxml()
+        fnode = tnode.find_first_by_tag("skipped")
+        fnode.assert_attr(message="xfail-marked test passes unexpectedly")
+        # assert "ValueError" in fnode.toxml()
 
     def test_collect_error(self, testdir):
         testdir.makepyfile("syntax error")
         result, dom = runandparse(testdir)
         assert result.ret
-        node = dom.getElementsByTagName("testsuite")[0]
-        assert_attr(node, errors=1, tests=0)
-        tnode = node.getElementsByTagName("testcase")[0]
-        assert_attr(tnode,
+        node = dom.find_first_by_tag("testsuite")
+        node.assert_attr(errors=1, tests=0)
+        tnode = node.find_first_by_tag("testcase")
+        tnode.assert_attr(
             file="test_collect_error.py",
-            #classname="test_collect_error",
             name="test_collect_error")
-        assert tnode.getAttributeNode("line") is None
-        fnode = tnode.getElementsByTagName("error")[0]
-        assert_attr(fnode, message="collection failure")
+        assert tnode["line"] is None
+        fnode = tnode.find_first_by_tag("error")
+        fnode.assert_attr(message="collection failure")
         assert "SyntaxError" in fnode.toxml()
 
     def test_collect_skipped(self, testdir):
         testdir.makepyfile("import pytest; pytest.skip('xyz')")
         result, dom = runandparse(testdir)
         assert result.ret == EXIT_NOTESTSCOLLECTED
-        node = dom.getElementsByTagName("testsuite")[0]
-        assert_attr(node, skips=1, tests=0)
-        tnode = node.getElementsByTagName("testcase")[0]
-        assert_attr(tnode,
+        node = dom.find_first_by_tag("testsuite")
+        node.assert_attr(skips=1, tests=0)
+        tnode = node.find_first_by_tag("testcase")
+        tnode.assert_attr(
             file="test_collect_skipped.py",
-            #classname="test_collect_error",
             name="test_collect_skipped")
-        assert tnode.getAttributeNode("line") is None # py.test doesn't give us a line here.
-        fnode = tnode.getElementsByTagName("skipped")[0]
-        assert_attr(fnode, message="collection skipped")
+
+        # py.test doesn't give us a line here.
+        assert tnode["line"] is None
+
+        fnode = tnode.find_first_by_tag("skipped")
+        fnode.assert_attr(message="collection skipped")
 
     def test_unicode(self, testdir):
         value = 'hx\xc4\x85\xc4\x87\n'
@@ -323,8 +378,8 @@ class TestPython:
         """ % value)
         result, dom = runandparse(testdir)
         assert result.ret == 1
-        tnode = dom.getElementsByTagName("testcase")[0]
-        fnode = tnode.getElementsByTagName("failure")[0]
+        tnode = dom.find_first_by_tag("testcase")
+        fnode = tnode.find_first_by_tag("failure")
         if not sys.platform.startswith("java"):
             assert "hx" in fnode.toxml()
 
@@ -347,9 +402,9 @@ class TestPython:
                 print('hello-stdout')
         """)
         result, dom = runandparse(testdir)
-        node = dom.getElementsByTagName("testsuite")[0]
-        pnode = node.getElementsByTagName("testcase")[0]
-        systemout = pnode.getElementsByTagName("system-out")[0]
+        node = dom.find_first_by_tag("testsuite")
+        pnode = node.find_first_by_tag("testcase")
+        systemout = pnode.find_first_by_tag("system-out")
         assert "hello-stdout" in systemout.toxml()
 
     def test_pass_captures_stderr(self, testdir):
@@ -359,27 +414,61 @@ class TestPython:
                 sys.stderr.write('hello-stderr')
         """)
         result, dom = runandparse(testdir)
-        node = dom.getElementsByTagName("testsuite")[0]
-        pnode = node.getElementsByTagName("testcase")[0]
-        systemout = pnode.getElementsByTagName("system-err")[0]
+        node = dom.find_first_by_tag("testsuite")
+        pnode = node.find_first_by_tag("testcase")
+        systemout = pnode.find_first_by_tag("system-err")
         assert "hello-stderr" in systemout.toxml()
 
+    def test_setup_error_captures_stdout(self, testdir):
+        testdir.makepyfile("""
+            def pytest_funcarg__arg(request):
+                print('hello-stdout')
+                raise ValueError()
+            def test_function(arg):
+                pass
+        """)
+        result, dom = runandparse(testdir)
+        node = dom.find_first_by_tag("testsuite")
+        pnode = node.find_first_by_tag("testcase")
+        systemout = pnode.find_first_by_tag("system-out")
+        assert "hello-stdout" in systemout.toxml()
+
+    def test_setup_error_captures_stderr(self, testdir):
+        testdir.makepyfile("""
+            import sys
+            def pytest_funcarg__arg(request):
+                sys.stderr.write('hello-stderr')
+                raise ValueError()
+            def test_function(arg):
+                pass
+        """)
+        result, dom = runandparse(testdir)
+        node = dom.find_first_by_tag("testsuite")
+        pnode = node.find_first_by_tag("testcase")
+        systemout = pnode.find_first_by_tag("system-err")
+        assert "hello-stderr" in systemout.toxml()
+
+
 def test_mangle_testnames():
     from _pytest.junitxml import mangle_testnames
     names = ["a/pything.py", "Class", "()", "method"]
     newnames = mangle_testnames(names)
     assert newnames == ["a.pything", "Class", "method"]
 
+
 def test_dont_configure_on_slaves(tmpdir):
     gotten = []
+
     class FakeConfig:
         def __init__(self):
             self.pluginmanager = self
             self.option = self
+
         junitprefix = None
-        #XXX: shouldnt need tmpdir ?
+        # XXX: shouldnt need tmpdir ?
         xmlpath = str(tmpdir.join('junix.xml'))
         register = gotten.append
+
     fake_config = FakeConfig()
     from _pytest import junitxml
     junitxml.pytest_configure(fake_config)
@@ -408,14 +497,12 @@ class TestNonPython:
         testdir.tmpdir.join("myfile.xyz").write("hello")
         result, dom = runandparse(testdir)
         assert result.ret
-        node = dom.getElementsByTagName("testsuite")[0]
-        assert_attr(node, errors=0, failures=1, skips=0, tests=1)
-        tnode = node.getElementsByTagName("testcase")[0]
-        assert_attr(tnode,
-            #classname="test_collect_error",
-            name="myfile.xyz")
-        fnode = tnode.getElementsByTagName("failure")[0]
-        assert_attr(fnode, message="custom item runtest failed")
+        node = dom.find_first_by_tag("testsuite")
+        node.assert_attr(errors=0, failures=1, skips=0, tests=1)
+        tnode = node.find_first_by_tag("testcase")
+        tnode.assert_attr(name="myfile.xyz")
+        fnode = tnode.find_first_by_tag("failure")
+        fnode.assert_attr(message="custom item runtest failed")
         assert "custom item runtest failed" in fnode.toxml()
 
 
@@ -449,6 +536,7 @@ def test_nullbyte_replace(testdir):
     text = xmlf.read()
     assert '#x0' in text
 
+
 def test_invalid_xml_escape():
     # Test some more invalid xml chars, the full range should be
     # tested really but let's just thest the edges of the ranges
@@ -463,14 +551,13 @@ def test_invalid_xml_escape():
         unichr(65)
     except NameError:
         unichr = chr
-    invalid = (0x00, 0x1, 0xB, 0xC, 0xE, 0x19,
-                27, # issue #126
-               0xD800, 0xDFFF, 0xFFFE, 0x0FFFF) #, 0x110000)
-    valid = (0x9, 0xA, 0x20,) # 0xD, 0xD7FF, 0xE000, 0xFFFD, 0x10000, 0x10FFFF)
+    invalid = (0x00, 0x1, 0xB, 0xC, 0xE, 0x19, 27,  # issue #126
+               0xD800, 0xDFFF, 0xFFFE, 0x0FFFF)  # , 0x110000)
+    valid = (0x9, 0xA, 0x20, )
+    # 0xD, 0xD7FF, 0xE000, 0xFFFD, 0x10000, 0x10FFFF)
 
     from _pytest.junitxml import bin_xml_escape
 
-
     for i in invalid:
         got = bin_xml_escape(unichr(i)).uniobj
         if i <= 0xFF:
@@ -481,6 +568,7 @@ def test_invalid_xml_escape():
     for i in valid:
         assert chr(i) == bin_xml_escape(unichr(i)).uniobj
 
+
 def test_logxml_path_expansion(tmpdir, monkeypatch):
     home_tilde = py.path.local(os.path.expanduser('~')).join('test.xml')
 
@@ -494,6 +582,7 @@ def test_logxml_path_expansion(tmpdir, monkeypatch):
     xml_var = LogXML('$HOME%stest.xml' % tmpdir.sep, None)
     assert xml_var.logfile == home_var
 
+
 def test_logxml_changingdir(testdir):
     testdir.makepyfile("""
         def test_func():
@@ -505,6 +594,7 @@ def test_logxml_changingdir(testdir):
     assert result.ret == 0
     assert testdir.tmpdir.join("a/x.xml").check()
 
+
 def test_logxml_makedir(testdir):
     """--junitxml should automatically create directories for the xml file"""
     testdir.makepyfile("""
@@ -515,6 +605,7 @@ def test_logxml_makedir(testdir):
     assert result.ret == 0
     assert testdir.tmpdir.join("path/to/results.xml").check()
 
+
 def test_escaped_parametrized_names_xml(testdir):
     testdir.makepyfile("""
         import pytest
@@ -524,49 +615,57 @@ def test_escaped_parametrized_names_xml(testdir):
     """)
     result, dom = runandparse(testdir)
     assert result.ret == 0
-    node = dom.getElementsByTagName("testcase")[0]
-    assert_attr(node,
-        name="test_func[#x00]")
+    node = dom.find_first_by_tag("testcase")
+    node.assert_attr(name="test_func[#x00]")
+
 
 def test_unicode_issue368(testdir):
     path = testdir.tmpdir.join("test.xml")
     log = LogXML(str(path), None)
     ustr = py.builtin._totext("ВНИ!", "utf-8")
     from _pytest.runner import BaseReport
+
     class Report(BaseReport):
         longrepr = ustr
         sections = []
         nodeid = "something"
         location = 'tests/filename.py', 42, 'TestClass.method'
-    report = Report()
+
+    test_report = Report()
 
     # hopefully this is not too brittle ...
     log.pytest_sessionstart()
-    log._opentestcase(report)
-    log.append_failure(report)
-    log.append_collect_error(report)
-    log.append_collect_skipped(report)
-    log.append_error(report)
-    report.longrepr = "filename", 1, ustr
-    log.append_skipped(report)
-    report.longrepr = "filename", 1, "Skipped: 卡嘣嘣"
-    log.append_skipped(report)
-    report.wasxfail = ustr
-    log.append_skipped(report)
+    node_reporter = log._opentestcase(test_report)
+    node_reporter.append_failure(test_report)
+    node_reporter.append_collect_error(test_report)
+    node_reporter.append_collect_skipped(test_report)
+    node_reporter.append_error(test_report)
+    test_report.longrepr = "filename", 1, ustr
+    node_reporter.append_skipped(test_report)
+    test_report.longrepr = "filename", 1, "Skipped: 卡嘣嘣"
+    node_reporter.append_skipped(test_report)
+    test_report.wasxfail = ustr
+    node_reporter.append_skipped(test_report)
     log.pytest_sessionfinish()
 
 
 def test_record_property(testdir):
     testdir.makepyfile("""
-        def test_record(record_xml_property):
+        import pytest
+
+        @pytest.fixture
+        def other(record_xml_property):
+            record_xml_property("bar", 1)
+        def test_record(record_xml_property, other):
             record_xml_property("foo", "<1");
     """)
     result, dom = runandparse(testdir, '-rw')
-    node = dom.getElementsByTagName("testsuite")[0]
-    tnode = node.getElementsByTagName("testcase")[0]
-    psnode = tnode.getElementsByTagName('properties')[0]
-    pnode = psnode.getElementsByTagName('property')[0]
-    assert_attr(pnode, name="foo", value="<1")
+    node = dom.find_first_by_tag("testsuite")
+    tnode = node.find_first_by_tag("testcase")
+    psnode = tnode.find_first_by_tag('properties')
+    pnodes = psnode.find_by_tag('property')
+    pnodes[0].assert_attr(name="bar", value="1")
+    pnodes[1].assert_attr(name="foo", value="<1")
     result.stdout.fnmatch_lines('*C3*test_record_property.py*experimental*')
 
 
@@ -583,10 +682,89 @@ def test_random_report_log_xdist(testdir):
             assert i != 22
     """)
     _, dom = runandparse(testdir, '-n2')
-    suite_node = dom.getElementsByTagName("testsuite")[0]
+    suite_node = dom.find_first_by_tag("testsuite")
     failed = []
-    for case_node in suite_node.getElementsByTagName("testcase"):
-        if case_node.getElementsByTagName('failure'):
-            failed.append(case_node.getAttributeNode('name').value)
+    for case_node in suite_node.find_by_tag("testcase"):
+        if case_node.find_first_by_tag('failure'):
+            failed.append(case_node['name'])
 
     assert failed == ['test_x[22]']
+
+
+def test_runs_twice(testdir):
+    f = testdir.makepyfile('''
+        def test_pass():
+            pass
+    ''')
+
+    result, dom = runandparse(testdir, f, f)
+    assert 'INTERNALERROR' not in result.stdout.str()
+    first, second = [x['classname'] for x in dom.find_by_tag("testcase")]
+    assert first == second
+
+
+@pytest.mark.xfail(reason='hangs', run=False)
+def test_runs_twice_xdist(testdir):
+    pytest.importorskip('xdist')
+    f = testdir.makepyfile('''
+        def test_pass():
+            pass
+    ''')
+
+    result, dom = runandparse(
+        testdir, f,
+        '--dist', 'each', '--tx', '2*popen',)
+    assert 'INTERNALERROR' not in result.stdout.str()
+    first, second = [x['classname'] for x in dom.find_by_tag("testcase")]
+    assert first == second
+
+
+def test_fancy_items_regression(testdir):
+    # issue 1259
+    testdir.makeconftest("""
+        import pytest
+        class FunItem(pytest.Item):
+            def runtest(self):
+                pass
+        class NoFunItem(pytest.Item):
+            def runtest(self):
+                pass
+
+        class FunCollector(pytest.File):
+            def collect(self):
+                return [
+                    FunItem('a', self),
+                    NoFunItem('a', self),
+                    NoFunItem('b', self),
+                ]
+
+        def pytest_collect_file(path, parent):
+            if path.check(ext='.py'):
+                return FunCollector(path, parent)
+    """)
+
+    testdir.makepyfile('''
+        def test_pass():
+            pass
+    ''')
+
+    result, dom = runandparse(testdir)
+
+    assert 'INTERNALERROR' not in result.stdout.str()
+
+    items = sorted(
+        '%(classname)s %(name)s %(file)s' % x
+
+        for x in dom.find_by_tag("testcase"))
+    import pprint
+    pprint.pprint(items)
+    assert items == [
+        u'conftest a conftest.py',
+        u'conftest a conftest.py',
+        u'conftest b conftest.py',
+        u'test_fancy_items_regression a test_fancy_items_regression.py',
+        u'test_fancy_items_regression a test_fancy_items_regression.py',
+        u'test_fancy_items_regression b test_fancy_items_regression.py',
+        u'test_fancy_items_regression test_pass'
+        u' test_fancy_items_regression.py',
+    ]
