@@ -107,6 +107,15 @@ class TestParser:
         parser.addoption("--hello", action="store_true")
         ns = parser.parse_known_args(["x", "--y", "--hello", "this"])
         assert ns.hello
+        assert ns.file_or_dir == ['x']
+
+    def test_parse_known_and_unknown_args(self, parser):
+        parser.addoption("--hello", action="store_true")
+        ns, unknown = parser.parse_known_and_unknown_args(["x", "--y",
+                                                           "--hello", "this"])
+        assert ns.hello
+        assert ns.file_or_dir == ['x']
+        assert unknown == ['--y', 'this']
 
     def test_parse_will_set_default(self, parser):
         parser.addoption("--hello", dest="hello", default="x", action="store")
