@@ -1,7 +1,9 @@
 import pytest
 import os
 from _pytest.pytester import HookRecorder
-from _pytest.core import PluginManager
+from _pytest.config import PytestPluginManager
+from _pytest.main import EXIT_OK, EXIT_TESTSFAILED
+
 
 def test_make_hook_recorder(testdir):
     item = testdir.getitem("def test_func(): pass")
@@ -62,14 +64,12 @@ def test_parseconfig(testdir):
 
 def test_testdir_runs_with_plugin(testdir):
     testdir.makepyfile("""
-        pytest_plugins = "pytest_pytester"
+        pytest_plugins = "pytester"
         def test_hello(testdir):
             assert 1
     """)
     result = testdir.runpytest()
-    result.stdout.fnmatch_lines([
-        "*1 passed*"
-    ])
+    result.assert_outcomes(passed=1)
 
 
 def make_holder():
@@ -91,8 +91,8 @@ def make_holder():
 
 @pytest.mark.parametrize("holder", make_holder())
 def test_hookrecorder_basic(holder):
-    pm = PluginManager()
-    pm.hook._addhooks(holder, "pytest_")
+    pm = PytestPluginManager()
+    pm.addhooks(holder)
     rec = HookRecorder(pm)
     pm.hook.pytest_xyz(arg=123)
     call = rec.popcall("pytest_xyz")
@@ -112,12 +112,11 @@ def test_makepyfile_unicode(testdir):
         unichr = chr
     testdir.makepyfile(unichr(0xfffd))
 
-def test_inprocess_plugins(testdir):
-    class Plugin(object):
-        configured = False
-        def pytest_configure(self, config):
-            self.configured = True
-    plugin = Plugin()
-    testdir.inprocess_run([], [plugin])
-
-    assert plugin.configured
+def test_inline_run_clean_modules(testdir):
+    test_mod = testdir.makepyfile("def test_foo(): assert True")
+    result = testdir.inline_run(str(test_mod))
+    assert result.ret == EXIT_OK
+    # rewrite module, now test should fail if module was re-imported
+    test_mod.write("def test_foo(): assert False")
+    result2 = testdir.inline_run(str(test_mod))
+    assert result2.ret == EXIT_TESTSFAILED
