@@ -1,16 +1,24 @@
 """
 terminal reporting of the full testing process.
 """
-import pytest, py
+import collections
+import pytest
+import py
 import sys
 
+from _pytest.main import EXIT_NOTESTSCOLLECTED
 from _pytest.terminal import TerminalReporter, repr_pythonversion, getreportopt
+from _pytest.terminal import build_summary_stats_line, _plugin_nameversions
 from _pytest import runner
+import _pytest._pluggy as pluggy
 
 def basic_run_report(item):
     runner.call_and_report(item, "setup", log=False)
     return runner.call_and_report(item, "call", log=False)
 
+DistInfo = collections.namedtuple('DistInfo', ['project_name', 'version'])
+
+
 class Option:
     def __init__(self, verbose=False, fulltrace=False):
         self.verbose = verbose
@@ -37,6 +45,20 @@ def pytest_generate_tests(metafunc):
                          funcargs={'option': Option(fulltrace=True)})
 
 
+@pytest.mark.parametrize('input,expected', [
+    ([DistInfo(project_name='test', version=1)], ['test-1']),
+    ([DistInfo(project_name='pytest-test', version=1)], ['test-1']),
+    ([
+        DistInfo(project_name='test', version=1),
+        DistInfo(project_name='test', version=1)
+    ], ['test-1']),
+], ids=['normal', 'prefix-strip', 'deduplicate'])
+def test_plugin_nameversion(input, expected):
+    pluginlist = [(None, x) for x in input]
+    result = _plugin_nameversions(pluginlist)
+    assert result == expected
+
+
 class TestTerminal:
     def test_pass_skip_fail(self, testdir, option):
         testdir.makepyfile("""
@@ -130,8 +152,8 @@ class TestTerminal:
         ])
 
     def test_itemreport_directclasses_not_shown_as_subclasses(self, testdir):
-        a = testdir.mkpydir("a")
-        a.join("test_hello.py").write(py.code.Source("""
+        a = testdir.mkpydir("a123")
+        a.join("test_hello123.py").write(py.code.Source("""
             class TestClass:
                 def test_method(self):
                     pass
@@ -139,7 +161,7 @@ class TestTerminal:
         result = testdir.runpytest("-v")
         assert result.ret == 0
         result.stdout.fnmatch_lines([
-            "*a/test_hello.py*PASS*",
+            "*a123/test_hello123.py*PASS*",
         ])
         assert " <- " not in result.stdout.str()
 
@@ -153,7 +175,7 @@ class TestTerminal:
                 raise KeyboardInterrupt   # simulating the user
         """)
 
-        result = testdir.runpytest(*option.args)
+        result = testdir.runpytest(*option.args, no_reraise_ctrlc=True)
         result.stdout.fnmatch_lines([
             "    def test_foobar():",
             ">       assert 0",
@@ -176,7 +198,7 @@ class TestTerminal:
                 pass
         """)
 
-        result = testdir.runpytest()
+        result = testdir.runpytest(no_reraise_ctrlc=True)
         assert result.ret == 2
         result.stdout.fnmatch_lines(['*KeyboardInterrupt*'])
 
@@ -408,13 +430,13 @@ class TestTerminalFunctional:
         verinfo = ".".join(map(str, py.std.sys.version_info[:3]))
         result.stdout.fnmatch_lines([
             "*===== test session starts ====*",
-            "platform %s -- Python %s* -- py-%s -- pytest-%s" % (
+            "platform %s -- Python %s*pytest-%s*py-%s*pluggy-%s" % (
                 py.std.sys.platform, verinfo,
-                py.__version__, pytest.__version__),
+                pytest.__version__, py.__version__, pluggy.__version__),
             "*test_header_trailer_info.py .",
-            "=* 1 passed in *.[0-9][0-9] seconds *=",
+            "=* 1 passed*in *.[0-9][0-9] seconds *=",
         ])
-        if pytest.config.pluginmanager._plugin_distinfo:
+        if pytest.config.pluginmanager.list_plugin_distinfo():
             result.stdout.fnmatch_lines([
                 "plugins: *",
             ])
@@ -457,7 +479,9 @@ class TestTerminalFunctional:
         ])
         assert result.ret == 1
 
-        pytestconfig.pluginmanager.skipifmissing("xdist")
+        if not pytestconfig.pluginmanager.get_plugin("xdist"):
+            pytest.skip("xdist plugin not installed")
+
         result = testdir.runpytest(p1, '-v', '-n 1')
         result.stdout.fnmatch_lines([
             "*FAIL*test_verbose_reporting.py::test_fail*",
@@ -572,7 +596,7 @@ def test_traceconfig(testdir, monkeypatch):
     result.stdout.fnmatch_lines([
         "*active plugins*"
     ])
-    assert result.ret == 0
+    assert result.ret == EXIT_NOTESTSCOLLECTED
 
 
 class TestGenericReporting:
@@ -714,3 +738,85 @@ def test_terminal_summary(testdir):
         *==== hello ====*
         world
     """)
+
+
+def test_terminal_summary_warnings_are_displayed(testdir):
+    """Test that warnings emitted during pytest_terminal_summary are displayed.
+    (#1305).
+    """
+    testdir.makeconftest("""
+        def pytest_terminal_summary(terminalreporter):
+            config = terminalreporter.config
+            config.warn('C1', 'internal warning')
+    """)
+    result = testdir.runpytest('-rw')
+    result.stdout.fnmatch_lines([
+        '*C1*internal warning',
+        '*== 1 pytest-warnings in *',
+    ])
+
+
+@pytest.mark.parametrize("exp_color, exp_line, stats_arg", [
+    # The method under test only cares about the length of each
+    # dict value, not the actual contents, so tuples of anything
+    # suffice
+
+    # Important statuses -- the highest priority of these always wins
+    ("red", "1 failed", {"failed": (1,)}),
+    ("red", "1 failed, 1 passed", {"failed": (1,), "passed": (1,)}),
+
+    ("red", "1 error", {"error": (1,)}),
+    ("red", "1 passed, 1 error", {"error": (1,), "passed": (1,)}),
+
+    # (a status that's not known to the code)
+    ("yellow", "1 weird", {"weird": (1,)}),
+    ("yellow", "1 passed, 1 weird", {"weird": (1,), "passed": (1,)}),
+
+    ("yellow", "1 pytest-warnings", {"warnings": (1,)}),
+    ("yellow", "1 passed, 1 pytest-warnings", {"warnings": (1,),
+                                               "passed": (1,)}),
+
+    ("green", "5 passed", {"passed": (1,2,3,4,5)}),
+
+
+    # "Boring" statuses.  These have no effect on the color of the summary
+    # line.  Thus, if *every* test has a boring status, the summary line stays
+    # at its default color, i.e. yellow, to warn the user that the test run
+    # produced no useful information
+    ("yellow", "1 skipped", {"skipped": (1,)}),
+    ("green", "1 passed, 1 skipped", {"skipped": (1,), "passed": (1,)}),
+
+    ("yellow", "1 deselected", {"deselected": (1,)}),
+    ("green", "1 passed, 1 deselected", {"deselected": (1,), "passed": (1,)}),
+
+    ("yellow", "1 xfailed", {"xfailed": (1,)}),
+    ("green", "1 passed, 1 xfailed", {"xfailed": (1,), "passed": (1,)}),
+
+    ("yellow", "1 xpassed", {"xpassed": (1,)}),
+    ("green", "1 passed, 1 xpassed", {"xpassed": (1,), "passed": (1,)}),
+
+    # Likewise if no tests were found at all
+    ("yellow", "no tests ran", {}),
+
+    # Test the empty-key special case
+    ("yellow", "no tests ran", {"": (1,)}),
+    ("green", "1 passed", {"": (1,), "passed": (1,)}),
+
+
+    # A couple more complex combinations
+    ("red", "1 failed, 2 passed, 3 xfailed",
+        {"passed": (1,2), "failed": (1,), "xfailed": (1,2,3)}),
+
+    ("green", "1 passed, 2 skipped, 3 deselected, 2 xfailed",
+        {"passed": (1,),
+        "skipped": (1,2),
+        "deselected": (1,2,3),
+        "xfailed": (1,2)}),
+])
+def test_summary_stats(exp_line, exp_color, stats_arg):
+    print("Based on stats: %s" % stats_arg)
+    print("Expect summary: \"%s\"; with color \"%s\"" % (exp_line, exp_color))
+    (line, color) = build_summary_stats_line(stats_arg)
+    print("Actually got:   \"%s\"; with color \"%s\"" % (line, color))
+    assert line == exp_line
+    assert color == exp_color
